import { EntityFactory } from './entity-factory';
import { IFactoryContainerOptions } from './factory-container-options.interface';
import { Type } from './utils';
/**
 * @class FactoryContainer
 * @summary
 * A container object to perform the connection string injection
 * into all of the factories. Maps each of the factories to respective
 * entity class name from reflected metadata.
 */
export declare class FactoryContainer {
    /** @property factories: A map from EntityName to its factory instance */
    private factories;
    /**
     * Instantiate a factory container and inject into all of the options.Factories.
     *
     * @param options: A configuration options for the factory container;
     * @returns a configured instance of the factory container
     */
    static init(options: IFactoryContainerOptions): Promise<FactoryContainer>;
    /**
     * Get a factory instance from entityName
     *
     * @param entity: The entity to retrieve the factory for
     * @returns the factory F from the factories map on the container
     */
    getFactory<K>(entity: Type<K>, namespaceKey?: string): EntityFactory<K>;
}
