"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FactoryContainer = void 0;
var factory_for_decorator_1 = require("./factory-for.decorator");
/**
 * @class FactoryContainer
 * @summary
 * A container object to perform the connection string injection
 * into all of the factories. Maps each of the factories to respective
 * entity class name from reflected metadata.
 */
var FactoryContainer = /** @class */ (function () {
    function FactoryContainer() {
    }
    /**
     * Instantiate a factory container and inject into all of the options.Factories.
     *
     * @param options: A configuration options for the factory container;
     * @returns a configured instance of the factory container
     */
    FactoryContainer.init = function (options) {
        return __awaiter(this, void 0, void 0, function () {
            var container, factories;
            return __generator(this, function (_a) {
                container = new FactoryContainer();
                factories = new Map();
                /** Instantiate all of the factory classes passed in via config */
                options.factories.forEach(function (FactoryClass) {
                    var _a = factory_for_decorator_1.getFactoryFor(FactoryClass), entityName = _a.entityName, namespaceKey = _a.namespaceKey;
                    var factoryInstance = new FactoryClass(options.em, container);
                    if (!entityName) {
                        throw new Error("\n          Unable to get entity metadata for factory " + factoryInstance.constructor.name + ".\n          This is likely due to the factory not being decorated with the FactoryFor class decorator.\n        ");
                    }
                    /** Disallow duplicate setting */
                    var accessKey = entityName + "_" + namespaceKey;
                    var existingValue = factories.get(accessKey);
                    if (existingValue) {
                        throw new Error("\n          Found duplicate factory for " + entityName + ". If you want to use multiple\n          factories for an entity, please provide a namespace key to the factory decorators.\n          @FactoryFor(" + entityName + ", <...namespace key ...>)\n        ");
                    }
                    factories.set(accessKey, factoryInstance);
                });
                /** Set the factory map */
                container.factories = factories;
                return [2 /*return*/, container];
            });
        });
    };
    /**
     * Get a factory instance from entityName
     *
     * @param entity: The entity to retrieve the factory for
     * @returns the factory F from the factories map on the container
     */
    FactoryContainer.prototype.getFactory = function (entity, namespaceKey) {
        if (namespaceKey === void 0) { namespaceKey = ''; }
        var entityName = entity.name;
        if (!entityName) {
            throw new Error("\n        Unable to find constructor name for entity parameter.\n        This is likely because you did not pass in a class.\n      ");
        }
        var factory = this.factories.get(entityName + "_" + namespaceKey);
        if (factory !== undefined) {
            return factory;
        }
        throw new Error("\n      Unable to retrieve factory instance for key: " + entityName + ".\n      This could be due to the entity not being passed into the init() function\n      on instantiation.\n      Possible entities are: " + __spread(this.factories.keys()) + "\n    ");
    };
    return FactoryContainer;
}());
exports.FactoryContainer = FactoryContainer;
